# Generated by Django 5.1.2 on 2025-03-14 08:01

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("payme", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="paymetransactions",
            name="fiscal_data",
            field=models.JSONField(blank=True, null=True),
        ),
    ]
