# Generated by Django 5.2.5 on 2025-08-25 19:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("payme", "0003_alter_paymetransactions_fiscal_data"),
    ]

    operations = [
        migrations.AlterField(
            model_name="paymetransactions",
            name="account_id",
            field=models.CharField(max_length=256),
        ),
    ]
